@@ -46,8 +46,10 @@
 
 #include <qvbox.h>
 #include <qhbox.h>
+#include <qlayout.h>
 #include <qpopupmenu.h>
 #include <qfile.h>
+#include <qsize.h>
 
 #include <kpopupmenu.h>
 #include <kmenubar.h>
@@ -68,6 +70,7 @@
 #include <kcmenumngr.h>
 #include <kio/job.h>
 #include <kstatusbar.h>
+#include <kprogress.h>
 
 KSlovar *KSlovar::m_instance=0L;
 
@@ -77,8 +80,14 @@ KSlovar::KSlovar()
   KSlovar::m_instance=this;
   m_configDialog=new KSConfigDialog(this, "settings", Configuration::self());
 
+  /*if(Configuration::authorName().isEmpty())
+  {
+    KMessageBox::information(0, i18n("You can set your name and email in configuration."));
+  }*/
+
   new KStatusBar(this);
-  statusBar()->insertItem(i18n("Ready."), 0);
+  statusBar()->insertItem(i18n("Ready."), 0, 1);
+  statusBar()->setItemAlignment(0, AlignLeft);
 
   if(!QFile::exists(locateLocal("config", "kslovarrc", false)))
   {
@@ -103,51 +112,59 @@ KSlovar::KSlovar()
   addMenu();
   addToolbar();
 
-  QHBox * horiz = new QHBox(this);
+  QHBox *horiz = new QHBox(this);
 
   m_split = new QSplitter(horiz);
 
-  QVBox * vert = new QVBox(m_split);
-  vert->setMaximumWidth(200);
-  vert->setMinimumWidth(200);
+  QVBox *vert = new QVBox(m_split);
 
   m_search = new KSSearchLine(vert);
-  m_list = new KSListView( vert );
+  m_list = new KSListView(vert);
   m_list->addColumn("name");
   m_list->setColumnWidth(0, 193);
   m_list->setFullWidth(true);
   m_search->setList(m_list);
+  m_list->setSorting(-1);
 
   KSData::instance()->setMainList(m_list);
 
   m_browser=new KHTMLPart(m_split);
   m_browser->setEncoding("utf-8", true);
   m_browser->view()->viewport()->installEventFilter(this); //For searching with middle click
+
+  QValueList<int> sizes;
+  sizes << 200 << 800;
+
+  m_split->setSizes(sizes);
+
   slotClose();
 
-  connect( m_browser->browserExtension(), SIGNAL( openURLRequest( const KURL &, const KParts::URLArgs & ) ), this, SLOT( slotShowBrowser(const KURL &, const KParts::URLArgs &) ) );
-  connect(m_list, SIGNAL( selectionChanged(QListViewItem *)), this, SLOT( slotShowList(QListViewItem *) ) );
+  connect(m_browser->browserExtension(), SIGNAL(openURLRequest(const KURL &, const KParts::URLArgs &)), this, SLOT(slotShowBrowser(const KURL &, const KParts::URLArgs &)));
+  connect(m_list, SIGNAL(selectionChanged(QListViewItem *)), this, SLOT(slotShowList(QListViewItem *)));
   connect(m_list, SIGNAL(doubleClicked( QListViewItem *)), this, SLOT(slotEditPhrase()));
   connect(kapp, SIGNAL(shutDown()), this, SLOT(slotClose()));
   connect(m_list, SIGNAL(contextMenu(KListView*, QListViewItem*, const QPoint&)), this, SLOT(showPopup( KListView*, QListViewItem*, const QPoint& )));
-  connect(m_list, SIGNAL(recievedPackage(bool, bool)), this, SLOT(slotCountPackages(bool, bool)));
 
   setCentralWidget(horiz);
 }
 
 void KSlovar::slotFileOpen()
 {
-  processFileOpen(KFileDialog::getOpenFileName(":dictionary", "*.ksd|KSlovar dictionary file", this));
+  processFileOpen(KFileDialog::getOpenFileName(":dictionary", "*.scmd|Self-Contained Multimedia Dictionary\n*.ksd|KSlovar Dictionary File (OLD)", this));
 }
 
 void KSlovar::showDictionary()
 {
-  m_history=false;
+  QString content = KSData::instance()->getDictionary()->processString("SELECT greeting FROM head LIMIT 1;")["greeting"];;
+  if (m_selectedPhrase.toInt() != 0)
+    content = XMLParser->parse(KSData::instance()->getDictionary()->processString("SELECT text FROM phrases WHERE id_phrase='"+m_selectedPhrase+"' LIMIT 1;")["text"]);
+
+  m_history = false;
   m_browser->begin();
-  m_browser->write(XMLParser->parse(KSData::instance()->getDictionary()->processString("SELECT text FROM dictionary WHERE id='"+m_selectedPhrase+"' LIMIT 1;")));
+  m_browser->write(content);
   m_browser->end();
 
-  if(!m_backHistory.isEmpty())
+  if (!m_backHistory.isEmpty())
   {
     m_back->setEnabled(true);
   }
@@ -206,19 +223,19 @@ void KSlovar::slotShowBrowser(const KURL &url, const KParts::URLArgs &)
 void KSlovar::slotPrevPhrase()
 {
   int temp = m_backHistory.first();
-  m_itForward = m_forwardHistory.prepend( m_selectedPhrase.toInt() );
+  m_itForward = m_forwardHistory.prepend(m_selectedPhrase.toInt());
 
   m_it = m_backHistory.remove(m_it);
 
-  if(m_backHistory.isEmpty())
+  if (m_backHistory.isEmpty())
   {
     m_back->setEnabled(false);
   }
   m_forward->setEnabled(true);
 
-  m_history=true;
+  m_history = true;
 
-  if(temp==0)
+  if (temp == 0)
   {
     m_list->clearSelection();
     m_editPhrase->setEnabled(false);
@@ -227,15 +244,15 @@ void KSlovar::slotPrevPhrase()
   }
   else
   {
-    QListViewItem *current=m_list->firstChild();
-    while(current)
+    QListViewItem *current = m_list->firstChild();
+    while (current)
     {
-      if(static_cast<KSListViewItem*> (current)->getId().toInt()==temp)
+      if (static_cast<KSListViewItem*> (current)->getId().toInt() == temp)
       {
         m_list->setSelected(current, true);
         break;
       }
-      current=current->nextSibling();
+      current = current->nextSibling();
     }
   }
 }
@@ -248,28 +265,38 @@ void KSlovar::slotNextPhrase()
 
   m_itForward = m_forwardHistory.remove(m_itForward);
 
-  if(m_forwardHistory.isEmpty())
+  if (m_forwardHistory.isEmpty())
   {
     m_forward->setEnabled(false);
   }
 
-  m_history=true;
+  m_history = true;
 
-  QListViewItem *current=m_list->firstChild();
-  while(current)
+  if (temp == 0)
   {
-    if(static_cast<KSListViewItem*> (current)->getId().toInt()==temp)
+    m_list->clearSelection();
+    m_editPhrase->setEnabled(false);
+    m_selectedPhrase.setNum(0);
+    showDictionary();
+  }
+  else
+  {
+    QListViewItem *current = m_list->firstChild();
+    while (current)
     {
-      m_list->setSelected(current, true);
-      break;
+      if (static_cast<KSListViewItem*> (current)->getId().toInt() == temp)
+      {
+        m_list->setSelected(current, true);
+        break;
+      }
+      current=current->nextSibling();
     }
-    current=current->nextSibling();
   }
 }
 
 void KSlovar::slotHome()
 {
-  if(m_selectedPhrase=="0")
+  if(m_selectedPhrase == "0")
   {
     return;
   }
@@ -306,7 +333,7 @@ void KSlovar::slotNewDictionary()
 
 void KSlovar::slotEditDictionary()
 {
-  QString text=KSData::instance()->getDictionary()->processString("SELECT text FROM dictionary WHERE id='0' LIMIT 1;");
+  QString text=KSData::instance()->getDictionary()->processString("SELECT greeting FROM head LIMIT 1;")["greeting"];
   QString name=text;
 
   text.remove(QRegExp("<h1>.+</h1>"));
@@ -328,12 +355,13 @@ void KSlovar::registerButtons()
   m_home=KStdAction::home(this, SLOT(slotHome()), actionCollection());
 
   m_editDictionary = new KAction(i18n("&Edit dictionary"), "edit", KShortcut(KKey("CTRL+e")), this, SLOT(slotEditDictionary()), actionCollection(), "editDictionary");
+  m_syncDictionary = new KAction(i18n("Synchronise dictionary's &parts of speech"), "reload", KShortcut(KKey("CTRL+p")), this, SLOT(slotSyncDictionary()), actionCollection(), "syncDictionary");
   m_close=KStdAction::close(this, SLOT(slotClose()), actionCollection());
 
   m_find = KStdAction::find(this, SLOT(slotFind()), actionCollection());
   m_findNext = KStdAction::findNext(this, SLOT(slotFindNext()), actionCollection());
-  m_literalSearch = new KToggleAction(i18n("&Literal search"), "filter", KShortcut(KKey("CTRL+l")), this, SLOT(slotToggleLiteral()), actionCollection(), "literalSearch");
-  m_backSearch = new KToggleAction(i18n("Ba&ck search"), "previous", KShortcut(KKey("CTRL+b")), this, SLOT(slotToggleBack()), actionCollection(), "backSearch");
+  /*m_literalSearch = new KToggleAction(i18n("&Literal search"), "filter", KShortcut(KKey("CTRL+l")), this, SLOT(slotToggleLiteral()), actionCollection(), "literalSearch");
+  m_backSearch = new KToggleAction(i18n("Ba&ck search"), "previous", KShortcut(KKey("CTRL+b")), this, SLOT(slotToggleBack()), actionCollection(), "backSearch");*/
 
   m_print=KStdAction::print(this, SLOT(slotPrint()), actionCollection());
   m_selectAll=KStdAction::selectAll(this, SLOT(slotSelectAll()), actionCollection());
@@ -366,6 +394,7 @@ void KSlovar::addMenu()
 
   KPopupMenu *editmenu=new KPopupMenu;
   m_editDictionary->plug(editmenu);
+  m_syncDictionary->plug(editmenu);
   m_selectAll->plug(editmenu);
   editmenu->insertSeparator();
   m_addPhrase->plug(editmenu);
@@ -374,8 +403,8 @@ void KSlovar::addMenu()
   editmenu->insertSeparator();
   m_find->plug(editmenu);
   m_findNext->plug(editmenu);
-  m_literalSearch->plug(editmenu);
-  m_backSearch->plug(editmenu);
+  /*m_literalSearch->plug(editmenu);
+  m_backSearch->plug(editmenu);*/
 
   KPopupMenu *gomenu=new KPopupMenu;
   m_back->plug(gomenu);
@@ -401,8 +430,8 @@ void KSlovar::addMenu()
 void KSlovar::addToolbar()
 {
   KToolBar *toolbar = new KToolBar(this);
-  m_literalSearch->plug(toolbar);
-  m_backSearch->plug(toolbar);
+  /*m_literalSearch->plug(toolbar);
+  m_backSearch->plug(toolbar);*/
   m_back->plug(toolbar);
   m_forward->plug(toolbar);
   m_home->plug(toolbar);
@@ -431,10 +460,10 @@ void KSlovar::slotClose()
 {
   if(KSData::instance()->getDictionary())
   {
-    KSData::instance()->getDictionary()->skip();
-    KSData::instance()->getDictionary()->terminate();
+    KSData::instance()->getDictionary()->stopThread();
   }
   disableNavButtons();
+  m_syncDictionary->setDisabled(true);
   m_backHistory.clear();
   m_forwardHistory.clear();
   m_selectedPhrase="";
@@ -444,7 +473,8 @@ void KSlovar::slotClose()
   m_list->setEmptyText(i18n("There is no dictionary loaded!"));
   m_list->clear();
   m_search->setDisabled(true);
-  m_search->setText("");
+  m_search->clear();
+  //m_search->setText("");
   statusBar()->changeItem(i18n("Ready"), 0);
 }
 
@@ -486,7 +516,7 @@ void KSlovar::slotEditPhrase()
   {
     return;
   }
-  QString output=KSData::instance()->getDictionary()->processString("SELECT text FROM dictionary WHERE id='"+m_selectedPhrase+"' LIMIT 1;");
+  QString output=KSData::instance()->getDictionary()->processString("SELECT text FROM phrases WHERE id_phrase='"+m_selectedPhrase+"' LIMIT 1;")["text"];
   m_phrasedlg=new KSPhrase(this, "Edit word");
   m_phrasedlg->setWord(output, m_selectedPhrase);
   m_phrasedlg->resize(700, 700);
@@ -497,15 +527,16 @@ void KSlovar::slotRemovePhrase()
 {
   bool generated = false;
   KSListViewItem *temp;
-  if(m_list->selectedItem())
+  /*if(m_list->selectedItem())
+  {*/
+  temp = static_cast<KSListViewItem*> (m_list->selectedItem());
+  //}
+  /*else
   {
-    temp = static_cast<KSListViewItem*> (m_list->selectedItem());
-  }
-  else
-  {
-    generated = true;
-    temp = new KSListViewItem(m_list, KSData::instance()->getDictionary()->processString("SELECT name FROM phrases WHERE id='" + m_selectedPhrase + "' LIMIT 1;"), m_selectedPhrase, KSData::instance()->getDictionary()->processString("SELECT search FROM phrases WHERE id='" + m_selectedPhrase + "' LIMIT 1;"));
-  }
+  generated = true;
+  KSResult tempResult = KSData::instance()->getDictionary()->processString("SELECT name, search FROM phrases WHERE id='" + m_selectedPhrase + "' LIMIT 1;")
+  temp = new KSListViewItem(m_list, , m_selectedPhrase, KSData::instance()->getDictionary()->processString("SELECT search FROM phrases WHERE id='" + m_selectedPhrase + "' LIMIT 1;"));
+}*/
 
   if(KMessageBox::questionYesNo(this, i18n("Are you really sure you want to delete " + temp->text(0) + "?"), i18n("Confirm deletion"), i18n("Delete")) == KMessageBox::No)
   {
@@ -516,13 +547,13 @@ void KSlovar::slotRemovePhrase()
     return;
   }
 
-  QString output = KSData::instance()->getDictionary()->processString("SELECT text FROM dictionary WHERE id='" + temp->getId() + "' LIMIT 1;");
+  QString output = KSData::instance()->getDictionary()->processString("SELECT text FROM phrases WHERE id_phrase='" + temp->getId() + "' LIMIT 1;")["text"];
 
   KSXMLHandler *XMLHandler = new KSXMLHandler(output.remove(QRegExp("<\\?.+\\?>")));
   QMap<QString,QString> synonyms = XMLHandler->readQMapList("synonym");
   for(QMap<QString,QString>::iterator count = synonyms.begin(); count != synonyms.end(); count++) //Removing synonyms
   {
-    QString deleting = KSData::instance()->getDictionary()->processString("SELECT text FROM dictionary WHERE id='" + count.key() + "' LIMIT 1;");
+    QString deleting = KSData::instance()->getDictionary()->processString("SELECT text FROM phrases WHERE id_phrase='" + count.key() + "' LIMIT 1;")["text"];
     KSXMLHandler *deleteHandler = new KSXMLHandler(deleting);
     deleteHandler->removeString("synonym", temp->text(0));
     KSData::instance()->getDictionary()->saveWord(count.data(), deleteHandler->parse(), false, count.key());
@@ -532,7 +563,7 @@ void KSlovar::slotRemovePhrase()
   QMap<QString,QString> antonyms = XMLHandler->readQMapList("antonym");
   for(QMap<QString,QString>::iterator count = antonyms.begin(); count != antonyms.end(); count++) //Removing antonyms
   {
-    QString deleting = KSData::instance()->getDictionary()->processString("SELECT text FROM dictionary WHERE id='" + count.key() + "' LIMIT 1;");
+    QString deleting = KSData::instance()->getDictionary()->processString("SELECT text FROM phrases WHERE id_phrase='" + count.key() + "' LIMIT 1;")["text"];
     KSXMLHandler *deleteHandler = new KSXMLHandler(deleting);
     deleteHandler->removeString("antonym", temp->text(0));
     KSData::instance()->getDictionary()->saveWord(count.data(), deleteHandler->parse(), false, count.key());
@@ -542,14 +573,14 @@ void KSlovar::slotRemovePhrase()
   QMap<QString,QString> wordFamily = XMLHandler->readQMapList("word-family");
   for(QMap<QString,QString>::iterator count = wordFamily.begin(); count != wordFamily.end(); count++) //Removing word family
   {
-    QString deleting = KSData::instance()->getDictionary()->processString("SELECT text FROM dictionary WHERE id='" + count.key() + "' LIMIT 1;");
+    QString deleting = KSData::instance()->getDictionary()->processString("SELECT text FROM phrases WHERE id_phrase='" + count.key() + "' LIMIT 1;")["text"];
     KSXMLHandler *deleteHandler = new KSXMLHandler(deleting);
     deleteHandler->removeString("word-family", temp->text(0));
     KSData::instance()->getDictionary()->saveWord(count.data(), deleteHandler->parse(), false, count.key());
     delete deleteHandler;
   }
 
-  KSData::instance()->getDictionary()->processQuery("DELETE FROM phrases WHERE id='"+temp->getId()+"'; DELETE FROM dictionary WHERE id='" + temp->getId() + "';");
+  KSData::instance()->getDictionary()->processQuery("BEGIN TRANSACTION; DELETE FROM phrases WHERE id_phrase='" + temp->getId() + "'; COMMIT;");
   delete temp;
   delete XMLHandler;
 
@@ -612,18 +643,70 @@ void KSlovar::processFileOpen(const QString &fileName)
     file.close();
 
     KSData::instance()->setDictionary(new KSDBHandler(fileName));
+
+    if(KSData::instance()->getDictionary()->processString("SELECT version FROM head;")["version"] != "0.1-rc3")
+    {
+      KMessageBox::error(this, i18n("The dictionary is too old. KSlovar will try to convert it into newer dictionary. It can fail! And expect some language break."));
+
+      // We get the data out
+      QString type = KSData::instance()->getDictionary()->processString("SELECT type FROM head;")["version"];
+      QString lang = KSData::instance()->getDictionary()->processString("SELECT lang FROM head;")["lang"];
+
+      // We try to guess the language name
+      lang = KSData::instance()->getLanguageName(lang.toInt());
+      QString text = KSData::instance()->getDictionary()->processString("SELECT text FROM dictionary WHERE id='0';")["text"];
+      QValueList<KSResult> oldPhrases = KSData::instance()->getDictionary()->processList("SELECT a.id, a.name, a.search, b.text FROM phrases a, dictionary b WHERE a.id=b.id;");
+
+      // Open the new dictionary
+      QString newFileName = fileName;
+      KSData::instance()->setDictionary(new KSDBHandler(newFileName.replace(".ksd", ".scmd")));
+
+      // We insert the data
+      KSData::instance()->getDictionary()->saveDictionary(text, lang, type, true);
+
+      QValueList<KSResult>::const_iterator end = oldPhrases.constEnd();
+      for (QValueList<KSResult>::const_iterator count = oldPhrases.constBegin(); count != end; count++)
+      {
+        KSData::instance()->getDictionary()->saveWord((*count)["name"], (*count)["text"], true);
+      }
+
+      KSData::instance()->setDictionary(0);
+
+      // Reopen the new dictionary
+      processFileOpen(newFileName);
+
+      // We quit this method
+      return;
+    }
+
+    // Progress bar
+    m_progress = new KProgress();
+    statusBar()->changeItem("Opening dictionary", 0);
+    statusBar()->addWidget(m_progress, 0);
+
+    m_progress->setTotalSteps(KSData::instance()->getDictionary()->processString("SELECT COUNT(id_phrase) AS number_phrases FROM phrases;")["number_phrases"].toInt());
+
     KSData::instance()->getDictionary()->start();
 
+
+    /*QValueList<KSResult> phrases = KSData::instance()->getDictionary()->processList("SELECT id, name, search FROM phrases;");
+    for(QValueList<KSResult>::iterator count = phrases.begin(); count != phrases.end(); count++)
+    {
+    new KSListViewItem(m_list, (*count)["name"], (*count)["id"], (*count)["search"]);
+  }*/
+
     m_home->setEnabled(true);
     m_editDictionary->setEnabled(true);
     m_close->setEnabled(true);
     m_addPhrase->setEnabled(true);
+    m_syncDictionary->setEnabled(true);
 
 
     m_backHistory.clear();
     KSData::instance()->setDictionaryPath(fileName);
-    loadPartOfSpeech(KSData::instance()->getDictionary()->processString("SELECT lang FROM head;").toInt());
-    int type = KSData::instance()->getDictionary()->processString("SELECT type FROM head;").toInt();
+    KSData::instance()->setLanguage(KSData::instance()->getLanguageId(KSData::instance()->getDictionary()->processString("SELECT lang FROM head;")["lang"]));
+    loadPartOfSpeech();
+    int type = KSData::instance()->getDictionary()->processString("SELECT type FROM head;")["type"].toInt();
     KSData::instance()->setType(type);
     if(!type)
     {
@@ -634,40 +717,40 @@ void KSlovar::processFileOpen(const QString &fileName)
       XMLParser=new KSXSLHandler(QString::fromUtf8(locate("appdata", "styles/"+Configuration::dictionaryStyle()+"/"+Configuration::dictionaryStyle()+"-transitional.xsl")));
     }
 
-    m_search->setDisabled(false);
-    m_list->setEmptyText(i18n("Begin search by typing a\nword into the search bar."));
-    m_list->clear();
+    m_list->setEmptyText(i18n("Loading words.\nPlease wait..."));
+    //m_list->clear();
     slotHome();
   }
 }
 
 void KSlovar::loadLanguages()
 {
-  if(!QFile::exists(locateLocal("appdata", "languages.ksl", false))) //Check if Languages.ksl exists. If not, run the upgrade manager to download it.
+  if(!QFile::exists(locateLocal("appdata", "languages.ldft", false))) //Check if Languages.ldft exists. If not, run the upgrade manager to download it.
   {
-    if(QFile::exists(locateLocal("appdata", "languages.ksl", false)+"~")) //Check if Languages.ksl exists. If not, run the upgrade manager to download it.
+    if(QFile::exists(locateLocal("appdata", "languages.ldft", false)+"~")) //Check if Languages.ldft exists. If not, run the upgrade manager to download it.
     {
-      KIO::move(KURL(locateLocal("appdata", "languages.ksl", false)+"~"), KURL(locateLocal("appdata", "languages.ksl", false)), false)->setInteractive(false);
+      KIO::move(KURL(locateLocal("appdata", "languages.ldft", false)+"~"), KURL(locateLocal("appdata", "languages.ldft", false)), false)->setInteractive(false);
       KIO::del(KURL(locateLocal("appdata", "version", false)), false, false)->setInteractive(false);
-      KMessageBox::information(this, i18n("Could not find languages.ksl. But found it's backup and using it."));
+      KMessageBox::information(this, i18n("Could not find languages.ldft. But found it's backup and using it."));
     }
     else
     {
       KIO::del(KURL(locateLocal("appdata", "version", false)), false, false)->setInteractive(false);
-      KMessageBox::error(this, i18n("Could not find languages.ksl. Run Upgrade manager to download it.\n\nIf you do not have any internet connection, you can download it from http://kslovar.berlios.de/languages.ksl and put it into ~/.kde/share/apps/kslovar/."));
+      KMessageBox::error(this, i18n("Could not find languages.ldft. Running first time wizard."));
+      slotFirstRunWizard();
       return;
     }
   }
-  KSData::instance()->setLanguageHandler(new KSDBHandler(QString::fromUtf8(locateLocal("appdata", "languages.ksl", false))));
-  QStringList input=KSData::instance()->getLanguageHandler()->processList("SELECT id, name FROM language;", 2);
+  KSData::instance()->setLanguageHandler(new KSDBHandler(QString::fromUtf8(locateLocal("appdata", "languages.ldft", false))));
+  QValueList<KSResult> input=KSData::instance()->getLanguageHandler()->processList("SELECT id_lang, name FROM language;");
   if(!input.isEmpty())
   {
-    QString id, name;
-    for(QStringList::iterator count=input.begin();count!=input.end();count++)
+    //QString id, name;
+    for(QValueList<KSResult>::iterator count = input.begin(); count != input.end(); count++)
     {
-      id=*count;
-      name=*count;
-      KSData::instance()->addLanguage(name.remove(QRegExp("^.+/")), id.remove(QRegExp("/.+$")).toInt());
+      /*id=*count;
+      name=*count;*/
+      KSData::instance()->addLanguage((*count)["name"], (*count)["id_lang"].toInt());
     }
   }
 
@@ -677,15 +760,15 @@ void KSlovar::loadLanguages()
   }
 
   input.clear();
-  input=KSData::instance()->getLanguageHandler()->processList("SELECT fromc, toc FROM conversion_table;", 2);
+  input = KSData::instance()->getLanguageHandler()->processList("SELECT fromc, toc FROM conversion_table;");
   if(!input.isEmpty())
   {
-    QChar from, to;
-    for(QStringList::iterator count=input.begin();count!=input.end();count++)
+    //QChar from, to;
+    for(QValueList<KSResult>::iterator count = input.begin(); count != input.end(); count++)
     {
-      from=(*count).at(0);
-      to=(*count).at(2);
-      KSData::instance()->addConversion(from, to);
+      /*from=(*count).at(0);
+      to=(*count).at(2);*/
+      KSData::instance()->addConversion((*count)["fromc"][0], (*count)["toc"][0]);
     }
   }
 }
@@ -693,16 +776,35 @@ void KSlovar::loadLanguages()
 void KSlovar::loadPartOfSpeech(int id)
 {
   KSData::instance()->clearPartOfSpeech();
-  KSData::instance()->setLanguage(id);
-  QStringList input=KSData::instance()->getLanguageHandler()->processList("SELECT id, name FROM type WHERE id_lang='"+QString::number(id)+"';", 2);
+  //KSData::instance()->setLanguage(id);
+  QValueList<KSResult> input = KSData::instance()->getLanguageHandler()->processList("SELECT id_type, name FROM type WHERE id_lang='"+QString::number(id)+"';");
+  //QValueList<KSResult> input = KSData::instance()->getDictionary()->processList("SELECT id_type, name FROM type;");
   if(!input.isEmpty())
   {
-    QString id, name;
-    for(QStringList::iterator count=input.begin();count!=input.end();count++)
+    //QString id, name;
+    for(QValueList<KSResult>::iterator count = input.begin(); count != input.end(); count++)
     {
-      id=*count;
-      name=*count;
-      KSData::instance()->addPartOfSpeech(name.remove(QRegExp("^.+/")), id.remove(QRegExp("/.+$")).toInt());
+      /*id=*count;
+      name=*count;*/
+      KSData::instance()->addPartOfSpeech((*count)["name"], (*count)["id_type"].toInt());
+    }
+  }
+}
+
+void KSlovar::loadPartOfSpeech()
+{
+  KSData::instance()->clearPartOfSpeech();
+  //KSData::instance()->setLanguage(id);
+  //QValueList<KSResult> input = KSData::instance()->getLanguageHandler()->processList("SELECT id_type, name FROM type WHERE id_lang='"+QString::number(id)+"';");
+  QValueList<KSResult> input = KSData::instance()->getDictionary()->processList("SELECT id_type, name FROM type;");
+  if(!input.isEmpty())
+  {
+    //QString id, name;
+    for(QValueList<KSResult>::iterator count = input.begin(); count != input.end(); count++)
+    {
+      /*id=*count;
+      name=*count;*/
+      KSData::instance()->addPartOfSpeech((*count)["name"], (*count)["id_type"].toInt());
     }
   }
 }
@@ -733,16 +835,6 @@ void KSlovar::slotDownloadLanguage()
   upgradeManager->show();
 }
 
-void KSlovar::slotToggleLiteral()
-{
-  KSData::instance()->setLiteralSearch(m_literalSearch->isChecked());
-}
-
-void KSlovar::slotToggleBack()
-{
-  KSData::instance()->setBackSearch(m_backSearch->isChecked());
-}
-
 void KSlovar::slotFirstRunWizard()
 {
   KSWizard *firstRun = new KSWizard(this, "firstRunWizard");
@@ -759,7 +851,7 @@ bool KSlovar::eventFilter(QObject *object, QEvent *event)
       if(m_list->isEnabled())
       {
         m_search->setText(selectedText);
-        m_search->processSearch();
+        m_search->manualSearch(selectedText);
       }
     }
   }
@@ -768,44 +860,49 @@ bool KSlovar::eventFilter(QObject *object, QEvent *event)
 
 void KSlovar::customEvent(QCustomEvent *package)
 {
-  if(package->type() == LIST)
+  KSOutputHandler *input = static_cast<KSOutputHandler*> (package);
+  switch(package->type())
   {
-    KSOutputHandler *input = static_cast<KSOutputHandler*> (package);
-    new KSListViewItem(m_list, input->getName(), input->getId(), input->getSearch());
+    case(LIST):
+    {
+      m_progress->setProgress(1);
+      m_progress->setTotalSteps(1);
+
+      QValueList<KSResult> phrases = input->getResult();
+      QValueList<KSResult>::const_iterator end = phrases.end();
+      for(QValueList<KSResult>::const_iterator count = phrases.begin(); count != end; count++)
+      {
+        new KSListViewItem(m_list, (*count)["name"], (*count)["id_phrase"], (*count)["search"]);
+      }
+      m_search->setDisabled(false);
+      statusBar()->changeItem(i18n("Ready"), 0);
+      delete m_progress;
+
+      m_list->setEmptyText(i18n("There are no words."));
+
+      break;
+    }
+    case(CONTROL):
+    {
+      m_progress->advance(input->getNumber());
+      break;
+    }
+    default:
+    {
+    }
   }
+
 }
 
-void KSlovar::slotCountPackages(bool found, bool completed)
+void KSlovar::slotSyncDictionary()
 {
-  if(found)
-  {
-    m_recievedPackages++;
-    statusBar()->changeItem(i18n("Found %1 words").arg(m_recievedPackages), 0);
-  }
-  else
-  {
-    m_recievedPackages = 0;
-    if(completed)
-    {
-      statusBar()->changeItem(i18n("No words found"), 0);
-    }
-    else
-    {
-      statusBar()->changeItem(i18n("Searching..."), 0);
-    }
-  }
-
+  KSData::instance()->getDictionary()->syncTypes();
 }
 
 KSlovar::~KSlovar()
 {
   delete KSData::instance()->getLanguageHandler();
   KSData::instance()->setLanguageHandler(0);
-  if(KSData::instance()->getDictionary())
-  {
-    KSData::instance()->getDictionary()->skip();
-    KSData::instance()->getDictionary()->terminate();
-  }
 }
 
 #include "kslovar.moc"
